--------------------------------------------------------------------------------
-- Execute the pExecStats procedure
--------------------------------------------------------------------------------

exec PerformanceDW.dbo.pExecStats 'R', 1 

--------------------------------------------------------------------------------
-- Create the pExecStats procedure
--
-- Procedure captured: November 2007
-- Be sure to check source for updates
-- http://statisticsio.com/Home/tabid/36/EntryID/4/Default.aspx
--
-- Code/comments below by Jason Massie (from statisticsio.com)
-- 9-20-2006 - Rev 1    
-- 9-20-2006 - Added physical reads to the average. Added xml plan option.  
--------------------------------------------------------------------------------
    
CREATE PROC pExecStats  
@sort char(1)=null,    
@xml bit = null  
as    
    
if @xml is null  
begin    
if (@sort) is null or (@sort) = 'C'    
select total_worker_time/execution_count as AvgCPU    
  , total_elapsed_time/execution_count as AvgDuration    
  , (total_logical_reads+total_physical_reads)/execution_count as AvgReads   
  , execution_count     
     , substring(st.text, (qs.statement_start_offset/2)+1    
                        , ((case qs.statement_end_offset    
                              when -1 then datalength(st.text)    
                              else qs.statement_end_offset    
                           end - qs.statement_start_offset)/2) + 1) as txt    
  from sys.dm_exec_query_stats as qs    
 cross apply sys.dm_exec_sql_text(qs.sql_handle) as st    
order by 1 desc    
    
if (@sort) = 'D'    
select total_worker_time/execution_count as AvgCPU    
  , total_elapsed_time/execution_count as AvgDuration    
  , (total_logical_reads+total_physical_reads)/execution_count as AvgReads  
  , execution_count     
     , substring(st.text, (qs.statement_start_offset/2)+1    
                        , ((case qs.statement_end_offset    
                              when -1 then datalength(st.text)    
                              else qs.statement_end_offset    
                           end - qs.statement_start_offset)/2) + 1) as txt    
  from sys.dm_exec_query_stats as qs    
 cross apply sys.dm_exec_sql_text(qs.sql_handle) as st    
order by 2 desc    
    
if (@sort) = 'R'    
select total_worker_time/execution_count as AvgCPU    
  , total_elapsed_time/execution_count as AvgDuration    
  , (total_logical_reads+total_physical_reads)/execution_count as AvgReads    
  , execution_count     
     , substring(st.text, (qs.statement_start_offset/2)+1    
                        , ((case qs.statement_end_offset    
                              when -1 then datalength(st.text)    
                              else qs.statement_end_offset    
                           end - qs.statement_start_offset)/2) + 1) as txt    
  from sys.dm_exec_query_stats as qs    
 cross apply sys.dm_exec_sql_text(qs.sql_handle) as st    
order by 3 desc    
    
if (@sort) = 'E'    
select total_worker_time/execution_count as AvgCPU    
  , total_elapsed_time/execution_count as AvgDuration    
  , (total_logical_reads+total_physical_reads)/execution_count as AvgReads  
  , execution_count     
     , substring(st.text, (qs.statement_start_offset/2)+1    
                        , ((case qs.statement_end_offset    
                              when -1 then datalength(st.text)    
                              else qs.statement_end_offset    
                           end - qs.statement_start_offset)/2) + 1) as txt    
  from sys.dm_exec_query_stats as qs    
 cross apply sys.dm_exec_sql_text(qs.sql_handle) as st    
order by 4 desc  
end  
else  
--xml showplan  
begin  
if (@sort) is null or (@sort) = 'C'    
select total_worker_time/execution_count as AvgCPU    
  , total_elapsed_time/execution_count as AvgDuration    
  , (total_logical_reads+total_physical_reads)/execution_count as AvgReads   
  , execution_count     
     , substring(st.text, (qs.statement_start_offset/2)+1    
                        , ((case qs.statement_end_offset    
                              when -1 then datalength(st.text)    
                              else qs.statement_end_offset    
                           end - qs.statement_start_offset)/2) + 1) as txt    
  , query_plan  
  from sys.dm_exec_query_stats as qs    
 cross apply sys.dm_exec_sql_text(qs.sql_handle) as st    
 cross apply sys.dm_exec_query_plan (qs.plan_handle) as qp   
order by 1 desc    
    
if (@sort) = 'D'    
select total_worker_time/execution_count as AvgCPU    
  , total_elapsed_time/execution_count as AvgDuration    
  , (total_logical_reads+total_physical_reads)/execution_count as AvgReads  
  , execution_count     
     , substring(st.text, (qs.statement_start_offset/2)+1    
                        , ((case qs.statement_end_offset    
                              when -1 then datalength(st.text)    
                              else qs.statement_end_offset    
                           end - qs.statement_start_offset)/2) + 1) as txt    
  , query_plan  
  from sys.dm_exec_query_stats as qs    
 cross apply sys.dm_exec_sql_text(qs.sql_handle) as st   
 cross apply sys.dm_exec_query_plan (qs.plan_handle) as qp   
order by 2 desc    
    
if (@sort) = 'R'    
select total_worker_time/execution_count as AvgCPU    
  , total_elapsed_time/execution_count as AvgDuration    
  , (total_logical_reads+total_physical_reads)/execution_count as AvgReads    
  , execution_count     
     , substring(st.text, (qs.statement_start_offset/2)+1    
                        , ((case qs.statement_end_offset    
                              when -1 then datalength(st.text)    
                              else qs.statement_end_offset    
                           end - qs.statement_start_offset)/2) + 1) as txt    
  , query_plan  
  from sys.dm_exec_query_stats as qs    
 cross apply sys.dm_exec_sql_text(qs.sql_handle) as st   
 cross apply sys.dm_exec_query_plan (qs.plan_handle) as qp    
order by 3 desc    
    
if (@sort) = 'E'    
select total_worker_time/execution_count as AvgCPU    
  , total_elapsed_time/execution_count as AvgDuration    
  , (total_logical_reads+total_physical_reads)/execution_count as AvgReads  
  , execution_count     
     , substring(st.text, (qs.statement_start_offset/2)+1    
                        , ((case qs.statement_end_offset    
                              when -1 then datalength(st.text)    
                              else qs.statement_end_offset    
                           end - qs.statement_start_offset)/2) + 1) as txt   
  , query_plan   
  from sys.dm_exec_query_stats as qs    
 cross apply sys.dm_exec_sql_text(qs.sql_handle) as st   
 cross apply sys.dm_exec_query_plan (qs.plan_handle) as qp    
order by 4 desc  
end  